@@ -34,16 +34,16 @@ def DistanceMatrix(A, method='shortest_unweighted_path'):
     np.fill_diagonal(A,0)
     A = A + A.T
 
-    if distance == 'shortest_unweighted_path':
+    if method == 'shortest_unweighted_path':
         D = network_tools.shortest_path(A, weighted_path_lengths = False) 
 
-    if distance == 'shortest_weighted_path':
+    if method == 'shortest_weighted_path':
         D = network_tools.shortest_path(A, weighted_path_lengths = True) 
 
-    if distance == 'weighted_shortest_path':
+    if method == 'weighted_shortest_path':
         D = network_tools.weighted_shortest_path(A)
 
-    if distance == 'diffusion_distance':
+    if method == 'diffusion_distance':
         G = nx.from_numpy_matrix(A)
         diam = nx.algorithms.distance_measures.diameter(G)
         walk_steps = int(2*diam)
